class BaseMagisterError(Exception):

    def __init__(self, message=None):
        super().__init__(message)
        self.message = message
        self.msg = message


class UnableToInputCredentials(BaseMagisterError):

    def __init__(self, message="\nCouldn't input the credentials\nThis error can occure if the credentials were not Input in order\nSchool->Username->Passwords"):
        super().__init__(message)
        self.message = message


class IncorrectCredentials(BaseMagisterError):
    def __init__(self, message="\nThe credentials provided were either incorrect or Magister rejected them"):
        super().__init__(message)
        self.message = message


class ConnectionError(BaseMagisterError):
    def __init__(self, message="\nCould not connect to Magister. Please check your internet connection"):
        super().__init__(message)
        self.message = message


class NotLoggedInError(BaseException):
    def __init__(self, message="You were not logged in before running this function"):
        super().__init__(message)
        self.message = message


class AuthcodeError(BaseMagisterError):
    def __init__(self, message="\nCould not get the authcode from the javascript. This is likely due to magister updating their code structure, please update the package or create an issue if it the latest version"):
        super().__init__(message)
        self.message = message


class FetchError(BaseMagisterError):
    def __init__(self, message="\nThere was an error fetching the data. The session has probably expired. Run relogin on the session or enable automatic_relogin through the session parameters."):
        super().__init__(message)
        self.message = message



from typing import Union


class JsonResponseItem():
    '''
    Base class for all of the response Items
    Makes sure that all of the dunder methods and all other methods that were not assigned are being applied to the inner json
    '''

    def __init__(self, json: Union[list, dict]):
        self.json = json
    # Apply all other methods to the original json

    def __getattr__(self, name):
        def wrapper(*args, **kwargs):
            return getattr(self.json, name)(*args, **kwargs)
        return wrapper

    def __getitem__(self, key):
        return self.json[key]

    def __setitem__(self, key, value):
        self.json[key] = value

    def __delitem__(self, key):
        del self.json[key]

    def __contains__(self, key):
        return key in self.json

    def __iter__(self):
        return iter(self.json)

    def __len__(self):
        return len(self.json)

    def __repr__(self):
        return f"JsonResponseItem({repr(self.json)})"


class Lesson(JsonResponseItem):
    '''
    A helper object that helps you interact with the retrieved lessons from the schedule.
    '''

    def __init__(self, json: dict, cancelled=False):
        '''
        Initializes a Lesson instance.

        Args:
            json (dict): The JSON data representing the lesson.
            cancelled (bool, optional): Indicates if the lesson is cancelled. Defaults to False.
        '''
        super().__init__(json=json)
        self.cancelled = cancelled  # Should be set by the get_schedule method

    def is_valid(self) -> bool:
        '''
        Checks if the lesson JSON contains all required keys.

        Returns:
            bool: True if all required keys are present, False otherwise.
        '''
        required_keys = {
            "Start", "Einde", "LesuurVan", "LesuurTotMet", "DuurtHeleDag",
            "Omschrijving", "Lokatie", "Status", "Type", "Subtype",
            "IsOnlineDeelname", "WeergaveType", "Inhoud", "Opmerking",
            "InfoType", "Aantekening", "Afgerond", "HerhaalStatus",
            "Herhaling", "Vakken", "Docenten", "Lokalen", "Groepen",
            "OpdrachtId", "HeeftBijlagen", "Bijlagen", "Id"
        }

        if not isinstance(self.json, dict):
            return False

        return not bool(required_keys - self.json.keys())

    def is_cancelled(self) -> bool:
        '''
        Checks if the lesson was marked as cancelled.

        Returns:
            bool: True if the lesson is cancelled, False otherwise.
        '''
        return self.cancelled

    def get_id(self) -> Union[None, int]:
        '''
        Retrieves the unique lesson ID.

        Returns:
            int: The ID of the lesson if available.
            None: If the ID is missing in the JSON.
        '''
        return self.json.get("Id")

    def get_location(self) -> Union[None, str]:
        '''
        Retrieves the location of the lesson.

        Returns:
            str: The location name if available.
            None: If the location is not found.
        '''
        return self.json.get("Lokatie")

    def get_start_time(self) -> Union[None, str]:
        '''
        Retrieves the lesson's start time.

        Returns:
            str: The start time if available.
            None: If the start time is missing.
        '''
        return self.json.get("Start")

    def get_end_time(self) -> Union[None, str]:
        '''
        Retrieves the lesson's end time.

        Returns:
            str: The end time if available.
            None: If the end time is missing.
        '''
        return self.json.get("Einde")

    def get_teacher_names(self) -> Union[None, list[dict]]:
        '''
        Retrieves the list of teachers associated with the lesson.

        Returns:
            list[dict]: A list of dictionaries containing teacher details.
            None: If no teachers are found.
        '''
        return self.json.get("Docenten")

    def get_locations(self) -> Union[None, list[str]]:
        '''
        Retrieves the names of all assigned locations.

        Returns:
            list[str]: A list of location names.
            None: If no valid locations exist.
        '''
        location_list = self.json.get("Lokalen")

        if not isinstance(location_list, list):
            return None

        return [location["Naam"] for location in location_list if "Naam" in location]

    def get_subject_names(self) -> Union[None, list[str]]:
        '''
        Retrieves the names of the subjects associated with the lesson.

        Returns:
            list[str]: A list of subject names.
            None: If no valid subjects exist.
        '''
        subject_list = self.json.get("Vakken")

        if not isinstance(subject_list, list):
            return None

        return [subject["Naam"] for subject in subject_list if "Naam" in subject]
    def get_description(self) -> Union[None,str]:
        '''
        Retrieves the description of the lesson.

        Returns:
            str: description string
            None: If no valid description exist.
        '''
        return self.json.get("Omschrijving")

class Grade(JsonResponseItem):
    '''
    A helper object that helps you interact with retrieved grade data.
    '''

    def __init__(self, json: Union[list, dict]):
        '''
        Initializes a Grade instance.

        Args:
            json (Union[list, dict]): The JSON data representing the grade.
        '''
        super().__init__(json)

    def is_valid(self) -> bool:
        '''
        Checks if the grade JSON contains all required keys.

        Returns:
            bool: True if all required keys are present, False otherwise.
        '''
        required_keys = {
            "kolomId", "omschrijving", "ingevoerdOp", "vak", "waarde",
            "weegfactor", "isVoldoende", "teltMee", "moetInhalen",
            "heeftVrijstelling", "behaaldOp", "links"
        }

        if not isinstance(self.json, dict):
            return False

        return not bool(required_keys - self.json.keys())

    def get_id(self) -> Union[None, int]:
        '''
        Retrieves the unique ID of the grade.

        Returns:
            int: The grade ID if available.
            None: If the ID is missing.
        '''
        return self.json.get("kolomId")

    def get_value(self) -> Union[None, str]:
        '''
        Retrieves the grade value.

        Returns:
            str: The grade as a string.
            None: If the value is missing.
        '''
        return self.json.get("waarde")

    def get_lesson_code(self) -> Union[None, str]:
        '''
        Retrieves the subject code associated with the grade.

        Returns:
            str: The subject code if available.
            None: If no valid subject code exists.
        '''
        grade_lesson_json = self.json.get("vak")

        if grade_lesson_json is not None:
            return grade_lesson_json.get("code")

        return None

    def get_lesson_name(self) -> Union[None, str]:
        '''
        Retrieves the subject name associated with the grade.

        Returns:
            str: The subject name if available.
            None: If no valid subject name exists.
        '''
        grade_lesson_json = self.json.get("vak")

        if grade_lesson_json is not None:
            return grade_lesson_json.get("omschrijving")

        return None

    def get_entered_time(self) -> Union[None, str]:
        '''
        Retrieves the time when the grade was entered.

        Returns:
            str: The timestamp of when the grade was recorded.
            None: If the timestamp is missing.
        '''
        return self.json.get("ingevoerdOp")

    def get_weighting_factor(self) -> Union[None, int]:
        '''
        Retrieves the weighting factor of the grade.

        Returns:
            int: The weighting factor if available.
            None: If the weighting factor is missing.
        '''
        return self.json.get("weegfactor")



import json
from .magister_errors import *


class JsParser():
    def __init__(self):
        pass

    def rfind_nth_instance(self, string: str, substring, n, start_from=-1):

        if start_from == -1:
            start_from = None

        index = string.rfind(substring, 0, start_from)

        if index != -1 and n > 1:
            return self.rfind_nth_instance(string, substring, n-1, index)
        return index

    def get_authcode_from_js(self, js_content: str):
        try:
            line = 1131

            buffer = 200
            authcode = ""
            # It is the only (for now) a string that appears right after the authcode obfuscation at line 1132
            authcode_identifier = "].map((function(t)"

            end_column = js_content.find(authcode_identifier)+1
            # Gets the last part of the authcode obfuscation
            js_content = js_content[end_column-buffer:end_column]

            start_column = self.rfind_nth_instance(js_content, "[", 2)
            js_content = js_content[start_column:]

            start_list_index = None
            content_list = []  # stores the 2 lists containing info about the authcode

            # Find the first and the second list
            for idx, _char in enumerate(js_content):

                if _char == "[":
                    start_list_index = idx

                if _char == "]" and (not (start_list_index is None)):

                    content_list.append(json.loads(
                        js_content[start_list_index:idx+1]))

                if len(content_list) > 1:
                    break

            def convert_to_int(a): return int(a)

            random_char_list, index_list = content_list

            index_list = list(map(convert_to_int, index_list))

            for idx in index_list:
                authcode += str(random_char_list[idx])

            return authcode
        except KeyboardInterrupt:
            raise KeyboardInterrupt()
        except Exception:
            raise AuthcodeError()



import sys
import os
import logging
sys.path.insert(0, os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../')))  # NOQA
import json
import unittest
from MagisterPy import JsParser
logging.basicConfig(level=logging.INFO)

# Replace with the actual module name where JsParser is defined


class TestJsParser(unittest.TestCase):
    def setUp(self):
        """Set up a JsParser instance for testing."""
        self.parser = JsParser()

    def test_valid_authcode_extraction(self):
        for filename in os.listdir("tests/test_javascripts/"):
            with open(f"tests/test_javascripts/{filename}") as file:
                content = file.read()
                logging.info(f"detected authcode: {self.parser.get_authcode_from_js(content)}") # NOQA

                self.assertEqual(True, True)


if __name__ == "__main__":
    unittest.main()



import requests
from .magister_errors import *


def should_start_relogin(_self):
    '''
    returns True if the error was caused outside of the relogin method and automatic relogin is enabled
    '''
    return _self.max_relogin_atempts == _self.relogin_atempts and _self.enable_automatic_relogin


def invoke_relogin(_self):
    if should_start_relogin(_self):
        return _self.relogin()


def error_handler(func, __recursive_call=None):
    '''
    A decorator used to handle errors that can occurre when when executing the methods
    '''
    if __recursive_call is None:
        __recursive_call = False

    def wrapper(*args, **kwargs):
        _self = args[0]
        errors_that_invoke_relogin = [FetchError,
                                      requests.exceptions.ConnectionError]
        try:
            result = func(*args, **kwargs)
            return result
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except (BaseMagisterError, requests.exceptions.ConnectionError) as e:
            # Detects if there is an irregular error in the method and makes sure that the method doesn't rerun recursively
            if any(isinstance(e, error_type) for error_type in errors_that_invoke_relogin) and not __recursive_call and invoke_relogin(_self):

                # Reruns the the method
                return error_handler(func, __recursive_call=True)(*args, **kwargs)

            elif not _self.automatically_handle_errors:
                raise e
            _self._logMessage(str(e))

    return wrapper



import unittest
import sys
import os
sys.path.insert(0, os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../')))  # NOQA
from MagisterPy import MagisterSession, Lesson, Grade


class TestLesson(unittest.TestCase):
    def setUp(self):
        self.valid_json = {
            "Start": "08:00", "Einde": "09:00", "LesuurVan": 1, "LesuurTotMet": 1,
            "DuurtHeleDag": False, "Omschrijving": "Math", "Lokatie": "Room 101",
            "Status": "Scheduled", "Type": "Lecture", "Subtype": "Normal",
            "IsOnlineDeelname": False, "WeergaveType": "Standard", "Inhoud": "Algebra",
            "Opmerking": "Bring calculator", "InfoType": "General", "Aantekening": "",
            "Afgerond": False, "HerhaalStatus": "None", "Herhaling": "None", "Vakken": [{"Naam": "Math"}, {"Naam": "Physics"}],
            "Docenten": [{"Naam": "Dr. Smith"}], "Lokalen": [{"Naam": "Room 101"}], "Groepen": [], "OpdrachtId": None,
            "HeeftBijlagen": False, "Bijlagen": [], "Id": 100
        }
        self.lesson = Lesson(self.valid_json)

    def test_is_valid(self):
        self.assertTrue(self.lesson.is_valid())

    def test_get_location(self):
        self.assertEqual(self.lesson.get_location(), "Room 101")

    def test_is_cancelled(self):
        self.assertFalse(self.lesson.is_cancelled())

    def test_get_start_time(self):
        self.assertEqual(self.lesson.get_start_time(), "08:00")

    def test_get_end_time(self):
        self.assertEqual(self.lesson.get_end_time(), "09:00")

    def test_get_teacher_names(self):
        self.assertEqual(self.lesson.get_teacher_names(),
                         [{"Naam": "Dr. Smith"}])

    def test_get_locations(self):
        self.assertEqual(self.lesson.get_locations(), ["Room 101"])

    def test_get_subject_names(self):
        self.assertEqual(self.lesson.get_subject_names(), ["Math", "Physics"])

    def test_get_id(self):
        self.assertEqual(self.lesson.get_id(), 100)
    def test_get_description(self):
        self.assertEqual(self.lesson.get_description(),"Math")

class TestGrade(unittest.TestCase):
    def setUp(self):
        self.valid_json = {
            "kolomId": 1, "omschrijving": "Exam", "ingevoerdOp": "2024-04-02",
            "vak": {"code": "MATH101", "omschrijving": "Mathematics"}, "waarde": "A",
            "weegfactor": 2, "isVoldoende": True, "teltMee": True, "moetInhalen": False,
            "heeftVrijstelling": False, "behaaldOp": "2024-04-01", "links": []
        }
        self.grade = Grade(self.valid_json)

    def test_is_valid(self):
        self.assertTrue(self.grade.is_valid())

    def test_get_value(self):
        self.assertEqual(self.grade.get_value(), "A")

    def test_get_lesson_code(self):
        self.assertEqual(self.grade.get_lesson_code(), "MATH101")

    def test_get_lesson_name(self):
        self.assertEqual(self.grade.get_lesson_name(), "Mathematics")

    def test_get_entered_time(self):
        self.assertEqual(self.grade.get_entered_time(), "2024-04-02")

    def test_get_weighting_factor(self):
        self.assertEqual(self.grade.get_weighting_factor(), 2)

    def test_get_id(self):
        self.assertEqual(self.grade.get_id(), 1)


if __name__ == "__main__":
    unittest.main()



import requests
from urllib.parse import urlparse, parse_qs
from .request_manager import LoginRequestsSender
from typing import Optional
from .error_handler import error_handler
from .magister_errors import *
import time
from .response_items import *


class MagisterSession():
    '''
    Creates a session with Magister

    Parameters:
            enable_logging (bool):  Used to display information in the standard output

            automatically_handle_errors (bool): Used to automatically handle errors. If any function fails it returns None instead of raising an error

            enable_automatic_relogin (bool): If set to True the session will atempt to relogin when it detects that a method has raised an error caused by session expiring or poor internet connection

            max_relogin_atempts (int): Specifies how many times the session will try to reconnect before throwing an error

            delay_between_relogin_atempts (int) How many seconds should transpire between each atempt.

    '''

    def __init__(self, enable_logging=False, automatically_handle_errors=True, max_relogin_atempts=5, enable_automatic_relogin=True, delay_between_relogin_atempts=2):

        self.request_sender = LoginRequestsSender()
        self.session = requests.Session()
        self.profile_auth_token = None  # auth token for the redirect page
        self.app_auth_token = None  # auth token for the main app
        self.authcode = None  # gets randomly generated once every 3-7 days
        self.sessionid = None  # gets assigned when entering the login page
        self.returnurl = None  # used for some requests
        # payload containing common parameters (authcode, returnurl, sessionid)
        self.main_payload = None
        self.person_id = None  # your account's person_id
        self.account_id = None  # your account id
        self.api_url = None  # url for accessing magister API
        self.x_correlation_id = None  # used in login requests
        self.automatically_handle_errors = automatically_handle_errors

        self.max_relogin_atempts = max_relogin_atempts
        self.relogin_atempts = max_relogin_atempts
        self.enable_automatic_relogin = enable_automatic_relogin
        self.delay_between_relogin_atempts = delay_between_relogin_atempts

        self.__school_name = None
        self.__username = None
        self.__password = None

        self.recieve_log = enable_logging

    def _logMessage(self, msg: str):
        if self.recieve_log:
            print(msg)

    @error_handler
    def input_school(self, school_name: str) -> Optional[requests.Response]:
        '''
        Sets up a session by inputting the school name. This is the **first step** in the login sequence 
        and must be called before `input_username()` and `input_password()`.

        Parameters:
            school_name (str): The name of the school to authenticate against.

        Usage:
            This method is the **first step** of the login process. It must be called before 
            `input_username()` and `input_password()` to initialize the session.

        Returns:
            requests.Response: if the school is found and session is successfully initiated.
            None: if the school is not found or there’s an issue in the school lookup.

        Example:
            session.input_school("MySchoolName")
        '''
        # clearing the session
        self.session = requests.Session()
        # Initializing the login session
        url_login_page = "https://accounts.magister.net/"

        response = self.session.get(url_login_page, allow_redirects=False)

        redirect_url_1 = r"https://accounts.magister.net/connect/authorize?client_id=iam-profile&redirect_uri=https%3A%2F%2Faccounts.magister.net%2Fprofile%2Foidc%2Fredirect_callback.html&response_type=id_token%20token&scope=openid%20profile%20email%20magister.iam.profile&state=57dcb9c3b667407791ff32a7af41e703&nonce=ec78d557c0e44751bf573db6719445cd"
        response = self.session.get(redirect_url_1, allow_redirects=False)

        response = self.session.get(response.headers.get(
            "location"), allow_redirects=False)

        response = self.session.get(
            "https://accounts.magister.net/" + response.headers.get("location"), allow_redirects=False)

        self.sessionid = parse_qs(urlparse(response.url).query).get(
            'sessionId', [None])[0]
        self.returnurl = parse_qs(urlparse(response.url).query).get(
            'returnUrl', [None])[0]
        self.x_correlation_id = parse_qs(urlparse(self.returnurl).query).get(
            'X-Correlation-ID', [None])[0]

        javascript_redirect_url = self.request_sender.extract_redirect_url_from_html(
            response.text)

        response = self.session.get(
            f"https://accounts.magister.net/{javascript_redirect_url}")

        self.authcode = self.request_sender.extract_dynamic_authcode(
            response.text)

        self.main_payload = {
            'authCode': self.authcode,
            'returnUrl': self.returnurl,
            'sessionId': self.sessionid
        }
        # Inputting the credentials

        # school
        try:
            response = self.request_sender.set_school(
                request_session=self.session, school_name=school_name, main_payload=self.main_payload)
            if response.status_code != 200:
                raise IncorrectCredentials(
                    f"Could not find school: {school_name}")
        except requests.exceptions.JSONDecodeError:
            raise IncorrectCredentials(f"Could not find school: {school_name}")
        self.__school_name = school_name
        return response

    @error_handler
    def input_username(self, username: str) -> Optional[requests.Response]:
        '''
    Sets the username for the current session. This is the **second step** in the login sequence 
    and must be called after `input_school()` but before `input_password()`.

    Parameters:
        username (str): The username for the account.

    Usage:
        This function must be called **after `input_school()`** and **before `input_password()`** to 
        authenticate the username.

    Returns:
        requests.Response: if the username is accepted.
        None: if the username is not found or if there’s an issue during authentication.

    Example:
        session.input_username("myusername")
    '''

        if not self.main_payload:
            raise UnableToInputCredentials()
        response = self.request_sender.set_username(
            request_session=self.session, username=username, main_payload=self.main_payload)
        if response.status_code != 200:
            raise IncorrectCredentials()
        self.__username = username
        return response

    @error_handler
    def input_password(self, password: str) -> Optional[requests.Response]:
        '''
        Sets the password for the session and finalizes the login process. This is the **third and final step** 
        in the login sequence and must be called after `input_school()` and `input_username()`.

        Upon success, this method retrieves and stores essential session variables like `profile_auth_token`, 
        `api_url`, `app_auth_token`, `account_id`, and `person_id` for further interactions with the API.

        Parameters:
            password (str): The password associated with the username.

        Usage:
            This function is the **final step** in the login process and should only be called **after 
            `input_school()` and `input_username()`**.

        Returns:
            requests.Response: if the password is correct and login is successful.
            None: if the password is incorrect or if there’s a login cooldown.

        Example:
            session.input_password("mypassword")
        '''
        if not self.main_payload:
            raise UnableToInputCredentials()
        response = self.request_sender.set_password(
            request_session=self.session, password=password, main_payload=self.main_payload)
        if response.status_code != 200:
            raise IncorrectCredentials(
                "Incorrect password or the password input is on cooldown")

        # setup for variables
        self.profile_auth_token = self.request_sender.get_profile_auth_token(
            request_session=self.session)
        self.api_url = self.request_sender.get_api_url(
            request_session=self.session, profile_auth_token=self.profile_auth_token)
        self.app_auth_token = self.request_sender.get_app_auth_token(
            request_session=self.session, api_url=self.api_url)
        self.account_id = self.request_sender.get_accountid(
            request_session=self.session, app_auth_token=self.app_auth_token, api_url=self.api_url)
        self.person_id = self.request_sender.get_personid(
            request_session=self.session, app_auth_token=self.app_auth_token, api_url=self.api_url, account_id=self.account_id)

        self._logMessage("you have successfully logged in!")
        self.__password = password
        return response

    @error_handler
    def login(self, school_name: str, username: str, password: str) -> bool:
        '''
        logs the user into their account

        returns:
        True -> if user logged in successfully
        False -> if user wasn't able to login

        params:
        school_name -> a string of school name (not case sensitive. It sets the first school from the list that magister provides)
        username -> a string of a username (should be exact)
        password -> a string with the password of the user (should be exact)
        '''
        # Inputting the school
        input_school_response = self.input_school(school_name=school_name)

        if not input_school_response:
            return False

        # username
        input_username_response = self.input_username(username=username)
        if not input_username_response:
            return False
        # password
        input_password_response = self.input_password(password=password)
        if not input_password_response:
            return False

        return True

    def relogin(self):
        '''
        Tries to relogin using the previously provided credentials (for example using login or input_{...} methods)
        True -> if user logged in successfully

        raises an error if all the relogin atempts were used up

        Keep in mind that the errors raised by this method are not handled automatically
        '''
        self._logMessage("Atempting to relogin...")
        if None in (self.__school_name, self.__password, self.__username):
            raise NotLoggedInError()
        while self.relogin_atempts > 0:
            self._logMessage(f"Atempts left: {self.relogin_atempts}")
            self.relogin_atempts -= 1
            result = self.login(school_name=self.__school_name,
                                username=self.__username,
                                password=self.__password)
            if result:
                self.relogin_atempts = self.max_relogin_atempts
                return True

            time.sleep(self.delay_between_relogin_atempts)
        raise ConnectionError(
            "\nCould not reconect to your account after all of the atempts")

    @error_handler
    def get_schedule(self, _from: str, to: str, with_changes=False) -> list[Lesson]:
        '''
    Retrieves the user’s schedule within a specified date range.

    This method fetches all scheduled items between two dates, starting from `_from` to `to`.
    The session must be authenticated by calling `.login()` first.

    Parameters:
    - _from (str): Start date of the schedule period in "YYYY-MM-DD" format.
    - to (str): End date of the schedule period in "YYYY-MM-DD" format.
    - with_changes: Retrieves a schedule including the cancelled lessons. The cancelled lessons will be marked as cancelled. 
      You can check if the lesson is cancelled by running is_cancelled method on the returned Lesson object inside of the list
    Returns:
    - list[Lesson]: A list of dictionaries representing schedule items, each with detailed fields. 
      The schedule items are sorted chronologically from earliest to latest.

    Structure of Each Schedule Item:
    ```json
    {
        "Start": "datetime",                   # Start time of the scheduled item
        "Einde": "datetime",                   # End time of the scheduled item
        "LesuurVan": bool,                     # Boolean for lesson period start
        "LesuurTotMet": bool,                  # Boolean for lesson period end
        "DuurtHeleDag": bool,                  # Whether the event lasts all day
        "Omschrijving": str,                   # Description of the event
        "Lokatie": str,                        # Location of the event
        "Status": int,                         # Status code of the event
        "Type": int,                           # Type identifier of the event
        "Subtype": int,                        # Subtype identifier of the event
        "IsOnlineDeelname": bool,              # Whether online attendance is allowed
        "WeergaveType": int,                   # Display type identifier
        "Inhoud": str,                         # Content or details about the event
        "Opmerking": str,                      # Additional notes
        "InfoType": int,                       # Information type identifier
        "Aantekening": str,                    # Notes or annotations
        "Afgerond": bool,                      # Whether the event is completed
        "HerhaalStatus": int,                  # Repeat status identifier
        "Herhaling": None,                     # Repeat information (usually null)
        "Vakken": list[dict],                  # List of subjects related to the event
        "Docenten": list[dict],                # List of teachers associated with the event
        "Lokalen": list[dict],                 # List of rooms assigned for the event
        "Groepen": None,                       # Group information (usually null)
        "OpdrachtId": int,                     # Task ID if associated
        "HeeftBijlagen": bool,                 # Whether attachments are available
        "Bijlagen": None                       # Attachment information (usually null)
    }
    ```

    Example Usage:
    ```python
    session.get_schedule(_from="2024-11-10", to="2024-11-11")
    ```
    '''
        response_original_schedule = None
        if not self.app_auth_token:
            self._logMessage("You have not logged in yet")
            return

        params = {
            "status": 1,
            "tot": to,
            "van": _from
        }
        headers = {"authorization": self.app_auth_token}

        # gets the schedule without the cancelled lessons
        url = f"{self.api_url}/personen/{self.person_id}/afspraken"
        response = self.session.get(url=url, params=params, headers=headers)

        # gets the schedule with cancelled lessons
        if with_changes:
            del params["status"]
            url = f"{self.api_url}/personen/{self.person_id}/afspraken"
            response_original_schedule = self.session.get(
                url=url, params=params, headers=headers)
            if response_original_schedule.status_code == 200:
                response_original_schedule = response_original_schedule.json().get("Items")
            else:
                response_original_schedule = None
        if response.status_code == 200:

            response_json = response.json().get("Items")

            if response_json is None:
                raise FetchError()
            if not response_original_schedule is None and with_changes:
                # check for cancelled lessons by finding id's that don't exist in both responses and setting cancelled parameter to true if they are
                response_json_ids = {Lesson(lesson).get_id()
                                     for lesson in response_json}

                response_original_schedule = [
                    Lesson(lesson) for lesson in response_original_schedule]
                response_original_schedule_ids = {
                    lesson.get_id() for lesson in response_original_schedule}

                cancelled_ids = response_original_schedule_ids - response_json_ids

                for lesson in response_original_schedule:
                    if lesson.get_id() in cancelled_ids:
                        lesson.cancelled = True
                return response_original_schedule
            # return schedule without cancelled items
            return [Lesson(lesson) for lesson in response_json]
        raise FetchError()

    @error_handler
    def get_grades(self, top: int = 25, skip: int = 0) -> list[Grade]:
        '''
    Retrieves the most recent grades for the user.

    This method fetches the latest grades for the authenticated user.
    The session must be authenticated by calling `.login()` first.

    Parameters:
    - top (int): Number of grades to retrieve (default is 25).
    - skip (int): Number of grades to skip, for pagination (default is 0).

    Returns:
    - list[dict]: A list of dictionaries, each representing a grade item with relevant details. 
      Grades are sorted from the most recent to the oldest.

    Structure of Each Grade Item:
    ```json
    {
        "omschrijving": str,                  # Description of the grade item
        "ingevoerdOp": "datetime",            # Date when the grade was entered
        "vak": {                              # Subject information
            "code": str,                      # Subject code
            "omschrijving": str               # Subject description
        },
        "waarde": str,                        # Grade value or score
        "weegfactor": float,                  # Weight factor of the grade
        "isVoldoende": bool,                  # Whether the grade is sufficient
        "teltMee": bool,                      # Whether the grade counts in the final score
        "moetInhalen": bool,                  # If the grade needs to be retaken
        "heeftVrijstelling": bool,            # If the grade has an exemption
        "behaaldOp": None,                    # Date achieved (if available)
        "links": dict                         # Additional links or references (usually empty)
    }
    ```

    Example Usage:
    ```python
    session.get_grades(top=1)
    ```
    '''
        if not self.app_auth_token:
            self._logMessage("You have not logged in yet")
            return

        params = {
            "top": top,
            "skip": skip
        }
        headers = {"authorization": self.app_auth_token}
        url = f"{self.api_url}/personen/{self.person_id}/cijfers/laatste"
        response = self.session.get(url=url, params=params, headers=headers)

        if response.status_code == 200:

            response_json = response.json().get("items")
            if response_json is None:
                raise FetchError()
            return [Grade(grade) for grade in response_json]

        raise FetchError()



import sys
import os
sys.path.insert(0, os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../')))  # NOQA
from MagisterPy import MagisterSession


school_name = "some_school"
username = "some_username"
password = "some_password"
# Create a new session and log in
session = MagisterSession(
    automatically_handle_errors=False, enable_logging=True, enable_automatic_relogin=True)

input_school_response = session.input_school(school_name=school_name)


input_username_response = session.input_username(username=username)


input_password_response = session.input_password(password=password)

# Get schedule for a specific date range
my_schedule = session.get_schedule("2024-11-03", "2024-11-10")

# Get the most recent grade
my_most_recent_grade = session.get_grades(top=1)[0]["waarde"]

print("Schedule in json:", my_schedule)
print("Most Recent Grade:", my_most_recent_grade)


# Simulate the session expiring
#
session.app_auth_token = "some_random_string"


# This should not cause an error
print(session.get_grades(top=1)[0]["waarde"])



from setuptools import setup, find_packages


def parse_requirements(filename):
    with open(filename, 'r') as f:
        return [line.strip() for line in f if line.strip() and not line.startswith('#')]


setup(
    name="MagisterPy",
    version="0.1.40",
    description="A Python package for retrieving information from magister",
    long_description=open("./README.MD").read(),
    long_description_content_type="text/markdown",
    author="H3LL0U",
    url="https://github.com/H3LL0U/MagisterPy",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
    include_package_data=True,
    zip_safe=False,
    install_requires=parse_requirements(
        './requirements.txt')  # Read dependencies
)



import requests
from urllib.parse import urlparse, parse_qs
from bs4 import BeautifulSoup
from .jsparser import *
from typing import Optional


class LoginRequestsSender():

    def __init__(self):
        pass

    def get_subdomain(self, url):

        parsed_url = urlparse(url)

        netloc = parsed_url.netloc

        parts = netloc.split('.')

        if len(parts) > 2:

            return '.'.join(parts[:-2])
        else:
            return None

    def get_accountid(self, request_session: requests.Session, app_auth_token, api_url) -> str:
        url = f"{api_url}/sessions/current"

        headers = {
            "authorization": app_auth_token
        }
        response = request_session.get(
            url, headers=headers, cookies=request_session.cookies)
        if response.status_code == 200:
            response_link = response.json()["links"]["account"]["href"]
            account_id = response_link[response_link.rfind("/")+1:]

            return account_id

    def get_personid(self, request_session: requests.Session, app_auth_token, api_url, account_id) -> str:

        url = f"{api_url}/accounts/{account_id}"

        headers = {
            "authorization": app_auth_token
        }

        response = request_session.get(url=url, headers=headers)

        if response.status_code == 200:
            response_link = response.json()["links"]["leerling"]["href"]
            personid = response_link[response_link.rfind("/")+1:]

            return personid

    def extract_auth_token(self, url) -> str:
        # Parse the URL to get the fragment
        parsed_url = urlparse(url)

        fragment = parsed_url.fragment

        # Parse the fragment to get the id_token
        token_params = parse_qs(fragment)
        id_token = token_params.get('access_token')

        if id_token:
            return id_token[0]  # Return the first id_token found
        else:
            return None

    def get_profile_auth_token(self, request_session: requests.Session) -> str:
        url = r"https://accounts.magister.net/connect/authorize?client_id=iam-profile&redirect_uri=https%3A%2F%2Faccounts.magister.net%2Fprofile%2Foidc%2Fredirect_callback.html&response_type=id_token%20token&scope=openid%20profile%20email%20magister.iam.profile&state=57dcb9c3b667407791ff32a7af41e703&nonce=ec78d557c0e44751bf573db6719445cd"
        response = request_session.get(url, allow_redirects=False)

        url = response.headers["Location"]
        return "Bearer " + self.extract_auth_token(url)

    def get_app_auth_token(self, request_session: requests.Session, api_url) -> str:
        url = "https://accounts.magister.net/connect/authorize"
        subdomain = self.get_subdomain(api_url)
        # Parameters as a dictionary
        params = {
            # might cause problems if subdomains for other schools are implemented differently TODO: Implement a system that reads https://{subdomain}.magister.net/oidc_config.js to get the string added to the domain at the client_id field
            "client_id": f"M6-{subdomain}.magister.net",
            "redirect_uri": f"https://{subdomain}.magister.net/oidc/redirect_callback.html",
            "response_type": "id_token token",
            "scope": "openid profile opp.read opp.manage attendance.overview attendance.administration "
            "calendar.user calendar.ical.user calendar.to-do.user grades.read grades.manage "
            "oso.administration registration.admin lockers.administration enrollment.admin",
            "state": "57dcb9c3b667407791ff32a7af41e703",
            "nonce": "ec78d557c0e44751bf573db6719445cd",
            "acr_values": f"tenant:{subdomain}.magister.net"
        }

        response = request_session.get(
            url, params=params, allow_redirects=False)
        url = response.headers["Location"]
        return "Bearer " + self.extract_auth_token(url)

    def get_api_url(self, request_session: requests.Session, profile_auth_token) -> str:
        headers = {"authorization": profile_auth_token}

        response = request_session.get(
            "https://magister.net/.well-known/host-meta.json", headers=headers)

        items = response.json()
        main_page = items["links"][0]["href"]
        return main_page

    def search_for_tenant_id(self, request_session: requests.Session, school_name, session_id) -> str:
        response = request_session.get(
            f"https://accounts.magister.net/challenges/tenant/search?sessionId={session_id}&key={school_name}")
        return response.json()[0]["id"]

    def set_school(self, request_session: requests.Session, school_name, main_payload) -> requests.Response:

        tenant_id = self.search_for_tenant_id(
            request_session, school_name, main_payload["sessionId"])
        main_payload["tenant"] = tenant_id

        response = self.send_post_request(
            request_session, main_payload, "https://accounts.magister.net/challenges/tenant")
        return response

    def set_password(self, request_session, password, main_payload) -> requests.Response:
        main_payload["password"] = password
        main_payload["userWantsToPairSoftToken"] = False
        return self.send_post_request(request_session, main_payload, "https://accounts.magister.net/challenges/password")

    def set_username(self, request_session, username, main_payload) -> requests.Response:

        main_payload["username"] = username
        response = self.send_post_request(
            request_session, main_payload, "https://accounts.magister.net/challenges/username")
        return response

    def send_post_request(self, request_session: requests.Session, payload, url, auth_token=None) -> requests.Response:

        headers = {
            "accept": "application/json",
            "content-type": "application/json",
            "authorization": auth_token,
            "origin": "https://accounts.magister.net",
            "x-xsrf-token": request_session.cookies.get("XSRF-TOKEN")
        }

        response = request_session.post(
            url=url, json=payload, headers=headers, cookies=request_session.cookies)
        return response

    def extract_redirect_url_from_html(self, html_content: str) -> str:
        soup = BeautifulSoup(html_content, 'html.parser')

        # Find the script tag with defer attribute
        script_tag = soup.find('script', {'defer': 'defer'})

        # Extract the src attribute
        if script_tag and 'src' in script_tag.attrs:
            script_src = script_tag['src']
            return script_src
        else:
            return None

    def extract_dynamic_authcode(self, js_content):
        return JsParser().get_authcode_from_js(js_content=js_content)
